--[[
Author: Mantrid
https://fs.mantrid.net/
Copyright: All rights reserved
LEGAL NOTICE: This mod, and this file in particular, are only authorised for distribution by GIANTS Software. All other entities agree to pay the sum of AU$1,000,000 for distribution rights.
Aspiring developers: I have limited experience with coding LUA, so my methods may not be the best. You're welcome to read and play with the code, for the purposes of learning, but do not copy whole functions for your own projects.
Experienced developers: It's my first day :)
]]

superPerson = {}

superPerson.buttonId = {}

superPerson.EDCRunning = false

function superPerson:loadMap(savegame)
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, superPerson.updateControlsMenu)
	Enterable.onRegisterActionEvents = Utils.appendedFunction(Enterable.onRegisterActionEvents, superPerson.updateControlsMenu)
	BaseMission.onLeaveVehicle = Utils.appendedFunction(BaseMission.onLeaveVehicle, superPerson.updateControlsMenu)
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, superPerson.removeActionEvents)
	FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, superPerson.saveSettings)
	Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, superPerson.loadSettings)

	Platform.gameplay.timeScaleSettings = { 0.5, 1, 2, 3, 5, 6, 10, 15, 30, 60, 120, 240, 360, 480, 600, 900, 1800, 3600, 5000, 10000, 20000, 30000 }
	
	-- If EDC is installed, we should disable the run speed there, when the player changes the run speed via this mod
	if FS22_EasyDevControls and FS22_EasyDevControls.EasyDevControls and FS22_EasyDevControls.EasyDevControls.setRunningSpeedActive then
		superPerson.EDCRunning = true
	end

	Logging.info("Super Person mod has been loaded")
end

function superPerson:callbackMenu_main(what, axisComponent)
	local motInfo = g_currentMission.player.motionInformation
	if what == "SUPER_PERSON_TOGGLEHUD" then
		if g_currentMission.hud.isVisible then
			g_currentMission.hud:setIsVisible(false)
		else
			g_currentMission.hud:setIsVisible(true)
		end
	elseif what == "SUPER_PERSON_TOGGLEFLIGHT" then
		Player.consoleCommandToggleFlightMode(g_currentMission.player)
		Player.onInputDebugFlyToggle(g_currentMission.player)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_RUNSPEED" and axisComponent == -1 then
		motInfo.maxRunningSpeed = MathUtil.clamp(motInfo.maxRunningSpeed - 5, 2, 200)
		motInfo.maxCheatRunningSpeed = MathUtil.clamp(motInfo.maxCheatRunningSpeed - 5, 2, 200)
		superPerson:updateControlsMenu()
		if superPerson.EDCRunning then FS22_EasyDevControls.EasyDevControls:setRunningSpeedActive(false) end
	elseif what == "SUPER_PERSON_RUNSPEED" and axisComponent == 1 then
		motInfo.maxRunningSpeed = MathUtil.clamp(motInfo.maxRunningSpeed + 5, 2, 200)
		motInfo.maxCheatRunningSpeed = MathUtil.clamp(motInfo.maxCheatRunningSpeed + 5, 2, 200)
		superPerson:updateControlsMenu()
		if superPerson.EDCRunning then FS22_EasyDevControls.EasyDevControls:setRunningSpeedActive(false) end
	elseif what == "SUPER_PERSON_JUMPHEIGHT" and axisComponent == -1 then
		motInfo.jumpHeight = MathUtil.clamp(motInfo.jumpHeight - 1, 2, 50)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_JUMPHEIGHT" and axisComponent == 1 then
		motInfo.jumpHeight = MathUtil.clamp(motInfo.jumpHeight + 1, 2, 50)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_SUPERSTRENGTH" then
		if g_currentMission.player.superStrengthEnabled then
			g_currentMission.player.superStrengthEnabled = false
			Player.MAX_PICKABLE_OBJECT_MASS = g_currentMission.player.superStrengthPickupMassBackup
			Player.MAX_PICKABLE_OBJECT_DISTANCE = g_currentMission.player.superStrengthPickupDistanceBackup
			g_currentMission.player.superStrengthPickupMassBackup = nil
		else
			g_currentMission.player.superStrengthEnabled = true
			g_currentMission.player.superStrengthPickupMassBackup = Player.MAX_PICKABLE_OBJECT_MASS
			Player.MAX_PICKABLE_OBJECT_MASS = 500
			g_currentMission.player.superStrengthPickupDistanceBackup = Player.MAX_PICKABLE_OBJECT_DISTANCE
			Player.MAX_PICKABLE_OBJECT_DISTANCE = 12
		end
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_FLIGHTHELP" then
	else
		Logging.warning("Unknown menuCallback: " .. tostring(what))
	end
end

function superPerson:updateControlsMenu()
	local motInfo = g_currentMission.player.motionInformation
	-- Remove all of our actions
	for key, eventId in pairs(superPerson.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		superPerson.buttonId[key] = nil
	end

	local textEnabled = g_i18n:getText("fieldJob_active")
	local textDisabled = g_i18n:getText("ui_production_status_inactive")

	local stateFlightMode = textDisabled
	local stateSuperStrength = textDisabled
	local stateRunningSpeed
	if g_addCheatCommands then
		stateRunningSpeed = motInfo.maxCheatRunningSpeed
	else
		stateRunningSpeed = motInfo.maxRunningSpeed
	end

	if g_flightModeEnabled then stateFlightMode = textEnabled end
	if g_currentMission.player.superStrengthEnabled then stateSuperStrength = textEnabled end

	local actions = {
		{ "SUPER_PERSON_TOGGLEHUD", "callbackMenu_main", g_i18n:getText("superPerson_menu_toggleHUD"), 5 }
	}

	if g_currentMission.controlledVehicle == nil then
		table.insert(actions, { "SUPER_PERSON_RUNSPEED", "callbackMenu_main", g_i18n:getText("superPerson_menu_runSpeed") .. " (" .. stateRunningSpeed .. ")", 6 })

		table.insert(actions, { "SUPER_PERSON_TOGGLEFLIGHT", "callbackMenu_main", g_i18n:getText("superPerson_menu_toggleFlight") .. " (" .. stateFlightMode .. ")", 7 })
		table.insert(actions, { "SUPER_PERSON_JUMPHEIGHT", "callbackMenu_main", g_i18n:getText("superPerson_menu_jumpHeight") .. " (" .. motInfo.jumpHeight .. ")", 6 })
		table.insert(actions, { "SUPER_PERSON_SUPERSTRENGTH", "callbackMenu_main", g_i18n:getText("superPerson_menu_superStrength") .. " (" .. stateSuperStrength .. ")", 6 })
		if g_flightModeEnabled then
			table.insert(actions, { "SUPER_PERSON_FLIGHTHELP", "callbackMenu_main", g_i18n:getText("superPerson_menu_flightHelp"), 8 })
		end
	end

	for _, item in ipairs(actions) do
		local _, eventId = nil
		if superPerson.buttonId[item[3]] == nil then -- Skip if menu item already exists.. even though it shouldn't
			_, eventId = g_inputBinding:registerActionEvent(InputAction[item[1]], superPerson, superPerson[item[2]], false, true, false, true)
			superPerson.buttonId[item[3]] = eventId
			g_inputBinding:setActionEventActive(eventId, true)
			g_inputBinding:setActionEventText(eventId, item[3])
			g_inputBinding:setActionEventTextPriority(eventId, item[4])
			g_inputBinding:setActionEventTextVisibility(eventId, true)
		end
	end
end

function superPerson:removeActionEvents()
	local g_inputBinding = g_inputBinding
	-- Remove all of our actions
	for key, eventId in pairs(superPerson.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		superPerson.buttonId[key] = nil
	end
end

function superPerson:saveSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	local motInfo = g_currentMission.player.motionInformation
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "superPerson"
	local xmlFile = createXMLFile(key, savegameFolderPath .. "/mod_superPerson.xml", key)

	local skyBrightness, _, _ = g_currentMission.environment.lighting.primaryExtraterrestrialColor:get(0)
	setXMLInt(xmlFile, key .. ".maxRunningSpeed#value", motInfo.maxRunningSpeed)
	setXMLInt(xmlFile, key .. ".maxCheatRunningSpeed#value", motInfo.maxCheatRunningSpeed)
	setXMLInt(xmlFile, key .. ".jumpHeight#value", motInfo.jumpHeight)
	setXMLBool(xmlFile, key .. ".superStrengthEnabled#value", Utils.getNoNil(g_currentMission.player.superStrengthEnabled, false))
	setXMLBool(xmlFile, key .. ".flightModeEnabled#value", g_flightModeEnabled)
	saveXMLFile(xmlFile)
	delete(xmlFile)
end
	
function superPerson:loadSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "superPerson"

	if fileExists(savegameFolderPath .. "/mod_superPerson.xml") then
		if consoleLogging then Logging.info("Super Person: Loading saved settings") end
		local xmlFile = loadXMLFile(key, savegameFolderPath .. "/mod_superPerson.xml")

		local maxRunningSpeed = getXMLFloat(xmlFile, key .. ".maxRunningSpeed#value")
		local maxCheatRunningSpeed = getXMLFloat(xmlFile, key .. ".maxCheatRunningSpeed#value")
		local jumpHeight = getXMLFloat(xmlFile, key .. ".jumpHeight#value")
		local superStrengthEnabled = getXMLBool(xmlFile, key .. ".superStrengthEnabled#value")
		local flightModeEnabled = getXMLBool(xmlFile, key .. ".flightModeEnabled#value")
		delete(xmlFile)
		
		local motInfo = g_currentMission.player.motionInformation
		
		if maxRunningSpeed ~= nil then
			motInfo.maxRunningSpeed = MathUtil.clamp(maxRunningSpeed, 2, 200)
		end
		if maxCheatRunningSpeed ~= nil then
			motInfo.maxCheatRunningSpeed = MathUtil.clamp(maxCheatRunningSpeed, 2, 200)
		end
		if jumpHeight ~= nil then
			motInfo.jumpHeight = MathUtil.clamp(jumpHeight, 2, 50)
		end
		if superStrengthEnabled ~= nil and superStrengthEnabled == true then
			g_currentMission.player.superStrengthEnabled = true
			g_currentMission.player.superStrengthPickupMassBackup = Player.MAX_PICKABLE_OBJECT_MASS
			Player.MAX_PICKABLE_OBJECT_MASS = 500
			g_currentMission.player.superStrengthPickupDistanceBackup = Player.MAX_PICKABLE_OBJECT_DISTANCE
			Player.MAX_PICKABLE_OBJECT_DISTANCE = 12
		end
		if flightModeEnabled ~= nil then
			if g_flightModeEnabled ~= flightModeEnabled then
				Player.consoleCommandToggleFlightMode(g_currentMission.player)
				Player.onInputDebugFlyToggle(g_currentMission.player)
			end
		end
		superPerson:updateControlsMenu()
	end
end


addModEventListener(superPerson)